# -*- coding: cp1252 -*-
"""Common filenames and locations for topographic data, meshes and outputs.
"""

from os import sep, environ, getenv, getcwd
from os.path import expanduser
import sys
from time import localtime, strftime, gmtime
from anuga.utilities.polygon import read_polygon, plot_polygons, polygon_area, is_inside_polygon

if sys.platform == 'win32':
    home = getenv('INUNDATIONHOME')
    user = getenv('USERPROFILE')

else:    
    home = getenv('INUNDATIONHOME', sep+'d'+sep+'xrd'+sep+'gem'+sep+'2'+sep+'ramp'+sep+'risk_assessment_methods_project'+sep+'inundation')     
    user = getenv('LOGNAME')
    print 'USER:', user

# INUNDATIONHOME is the inundation directory, not the data directory.
home += sep +'data'

#Making assumptions about the location of scenario data
state = 'new_south_wales'
scenario_dir_name = 'sydney_tsunami_scenario_2006'

# onshore data provided by NSW LPI, Krishna merged with existing
# PMD data to make 100m DEM
on_offshore_name25 = 'bathy_dem25'
on_offshore_name100 = 'bathyland100'
nsw100 = 'dem_utm'

# survey data
offshore_name1 = 'surveyAreaA'
offshore_name2 = 'surveyAreaB'
offshore_name3 = 'surveyAreaC'

# AHO data
offshore_name4 = '1000003611export'
offshore_name5 = '1000003613export'
offshore_name6 = '1000003614export'
offshore_name7 = '1000003627export'
offshore_name8 = '1000003628export'
offshore_name9 = 'AHDexport'

#swollen/ all data output
basename = 'source'
codename = 'project_slide.py'

#Derive subdirectories and filenames
local_time = strftime('%Y%m%d_%H%M%S',gmtime()) 
meshdir = home+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'meshes'+sep
datadir = home+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'topographies'+sep
gaugedir = home+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'gauges'+sep
polygondir = home+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'polygons'+sep
boundarydir = home+sep+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'boundaries'+sep
outputdir = home+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'outputs'+sep
outputtimedir = outputdir + local_time + sep
polygondir = home+sep+state+sep+scenario_dir_name+sep+'anuga'+sep+'polygons'+sep

gauge_filename = gaugedir + 'sydney_slide_gauges.csv'
codedir = getcwd()+sep                            
codedirname = codedir + 'project_slide.py'
meshname = outputtimedir + 'mesh_' + basename

# Necessary if using point datasets, rather than grid
on_offshore25_dem_name = datadir + on_offshore_name25
on_offshore100_dem_name = datadir + on_offshore_name100
nsw_dem_name = datadir + nsw100
offshore_dem_name1 = datadir + offshore_name1
offshore_dem_name2 = datadir + offshore_name2
offshore_dem_name3 = datadir + offshore_name3
offshore_dem_name4 = datadir + offshore_name4
offshore_dem_name5 = datadir + offshore_name5
offshore_dem_name6 = datadir + offshore_name6
offshore_dem_name7 = datadir + offshore_name7
offshore_dem_name8 = datadir + offshore_name8
offshore_dem_name9 = datadir + offshore_name9
combined_dem_name   = datadir + 'sydney_combined_elevation'

###############################
# Domain definitions
###############################

# bounding polygon for study area
polyAll = read_polygon(polygondir+'extent3.csv')

print 'Area of bounding polygon', polygon_area(polyAll)/1000000.0

###############################
# Interior region definitions
###############################

# sydney digitized polygons
poly_coast = read_polygon(polygondir+'coast2.csv')
poly_syd1 = read_polygon(polygondir+'local.csv')

print 'Area of local polygon', polygon_area(poly_syd1)/1000000.0
print 'Area of coastal polygon', polygon_area(poly_coast)/1000000.0

###################################################################
# Clipping regions for export to asc and regions for clipping data
###################################################################

# clipping for 25m grid for conversion
eastingmin25 = 324825
eastingmax25 = 347650
northingmin25 = 6231960
northingmax25 = 6272543

poly_25mclip = read_polygon(polygondir+'25mclip.csv')

# clipping for 100m grid for conversion
eastingmin100 = 315870
eastingmax100 = 361607
northingmin100 = 6227588
northingmax100 = 6285975

poly_surveyclip = read_polygon(polygondir+'surveyclip.csv')

# clipping for nsw grid for conversion
eastingmin_nsw = 315870
eastingmax_nsw = 390480
northingmin_nsw = 6161100 
northingmax_nsw = 6330000

poly_origsyd = [[eastingmin100,northingmin100],[eastingmax100,northingmin100],\
                [eastingmax100,northingmax100],[eastingmin100,northingmax100]]

# exporting asc grid
eastingmin = 333000
eastingmax = 354000
northingmin = 6244800
northingmax = 6274550

# exporting asc grid
eastingmin = 318922+10000
eastingmax = 318922+45000
northingmin = 6231749+7500
northingmax = 6231749+48000

#eastingmin = -40
#eastingmax = 40
#northingmin = -20
#northingmax = 20

###################################################################
# Slide characteristics
###################################################################

# historical slides
slide_origin_bulli = [370110,6189489]
slide_origin_shovel = [354753,6187397]
#slide_origin_yacaaba = [463012,6334735] original work was actually Birubi
slide_origin_yacaaba = [476465, 6362424]
slide_origin_birubi = [462267,6335100]

# potential slides: these correspond approximately to modelled depths
slide_origin_bulli_b = [405617,6249852]
slide_origin_bulli_c0 = [438583,6308510]
slide_origin_bulli_c1 = [465283,6335210]
slide_origin_bulli_c2 = [483783,6358810]
slide_origin_shovel_b = [396262,6252144]
slide_origin_shovel_c0 = [431683,6311110]
slide_origin_shovel_c1 = [458694,6340352]
slide_origin_shovel_c2 = [475589,6363030]
slide_origin_yacaaba_a = [359796,6190943]
slide_origin_yacaaba_b = [397330,6251857]
slide_origin_yacaaba_c0 = [431283,6312110]
slide_origin_yacaaba_c1 = [459983,6339110]
slide_origin_birubi_a = [362811,6191183]
slide_origin_birubi_b = [403573,6250268]
slide_origin_birubi_c0 = [437183,6309410]
slide_origin_birubi_c2 = [483392,6359044]

#bulli_depth = 2087.0
bulli_depth = 1470.0
bulli_length = 16840.0
bulli_thickness = 424.0
bulli_width = 8860.0
bulli_density = 1.46
bulli_slope = 4.0
bulli_alpha = 126.0 - 90.0

#shovel_depth = 968.0
shovel_depth = 877.0
shovel_length = 13500.0
shovel_thickness = 165.0
shovel_width = 4350.0
shovel_density = 1.49
shovel_slope = 4.0
shovel_alpha = 118.0 - 90.0

#yacaaba_depth = 1119.0
yacaaba_depth = 938.
yacaaba_length = 4189.
yacaaba_thickness = 53.
yacaaba_width = 2898.
yacaaba_density = 1.48
yacaaba_slope = 2.3
yacaaba_alpha = 133.0 - 90.0
 
birubi_depth = 1320.
birubi_length = 9903.0
birubi_width = 4150.
birubi_density = 1.48
birubi_slope = 3.7
birubi_alpha = 133.0 - 90.0
birubi_thickness = 140.
