import os
from math import sqrt, pi, sin, cos
from shallow_water_domain_suggestion3 import *
from Numeric import allclose, array, zeros, ones, Float, take, sqrt
from config import g, epsilon
import time

def analytic_cannal(C,t):
    N = len(C)
    u = zeros(N,Float)              ## water velocity
    h = zeros(N,Float)              ## water depth
    x = C
    g = 9.81
    ## Define Basin Bathymetry
    z_b = zeros(N,Float)            ## elevation of basin
    w = zeros(N,Float)              ## elevation of water surface
    D0 = 10.0                       ## max equilibrium water depth at lowest point.
    L_x = 2500.0                    ## width of channel
    A0 = 0.5*L_x                    ## determines amplitudes of oscillations
    omega = sqrt(2*g*D0)/L_x        ## angular frequency of osccilation
    for i in range(N):
        z_b[i] = D0*(x[i]**2/L_x**2)
        u[i] = -A0*omega*sin(omega*t)
        w[i] = D0+2*A0*D0/L_x*cos(omega*t)*(x[i]/L_x-0.5*A0/(L_x)*cos(omega*t))
        if w[i] <= z_b[i] :
            u[i] = 0.0
            w[i] = z_b[i]
    h = w - z_b
    T = 2.0*pi/omega
    return u,h,w,z_b, T


L_x = 2500.0                        # Length of channel (m)
N = 400                               # Number of computational cells
cell_len = 4*L_x/N                  # Origin = 0.0

points = zeros(N+1,Float)
for i in range(N+1):
        points[i] = -2*L_x +i*cell_len

domain = Domain(points)
domain.order = 2
domain.set_timestepping_method('rk2')
domain.set_CFL(1.0)
domain.beta = 1.0
domain.set_limiter("minmod")

def stage(x):
    D0 = 10.0                       ## max equilibrium water depth at lowest point.
    L_x = 2500.0                    ## width of channel
    A0 = 0.5*L_x                    ## determines amplitudes of oscillations
    omega = sqrt(2*g*D0)/L_x        ## angular frequency of osccilation
    t=0.0
    y = zeros(len(x),Float)
    for i in range(len(x)):
        y[i] = D0+2*A0*D0/L_x*cos(omega*t)*(x[i]/L_x-0.5*A0/(L_x)*cos(omega*t))
        #y[i] = 12.0
    return y

def elevation(x):
    N = len(x)
    D0 = 10.0
    z = zeros(N,Float)
    L_x = 2500.0
    A0 = 0.5*L_x
    omega = sqrt(2*g*D0)/L_x
    for i in range(N):
        z[i] = D0*(x[i]**2/L_x**2)
    return z

def height(x):
    D0 = 10.0                       ## max equilibrium water depth at lowest point.
    L_x = 2500.0                    ## width of channel
    A0 = 0.5*L_x                    ## determines amplitudes of oscillations
    omega = sqrt(2*g*D0)/L_x        ## angular frequency of osccilation
    t=0.0
    y = zeros(len(x),Float)
    for i in range(len(x)):
        y[i] = max(D0+2*A0*D0/L_x*cos(omega*t)*(x[i]/L_x-0.5*A0/(L_x)*cos(omega*t)) - D0*(x[i]**2/L_x**2),  0.0)
    return y


domain.set_quantity('stage', stage)
domain.set_quantity('elevation',elevation)
#domain.set_quantity('height',height)
domain.set_boundary({'exterior': Reflective_boundary(domain)})

C = domain.centroids
X = domain.vertices
u,h,w,z_b,T = analytic_cannal(X.flat,domain.time)
print 'T = ',T

yieldstep = finaltime = T/16 #3.0*T/2.0
StageQ = domain.quantities['stage'].vertex_values
XmomQ = domain.quantities['xmomentum'].vertex_values

import time
while finaltime < T + 1.0:
    yieldstep = finaltime
    t0 = time.time()
    for t in domain.evolve(yieldstep = yieldstep, finaltime = finaltime):
        domain.write_time()
        #print "integral", domain.quantities['stage'].get_integral()
        if t>0.0:
            print "t=",t
            uC,hC,wC,z_bC,TC = analytic_cannal(C,t)
            for k in range(len(hC)):
                if hC[k] < 0.0:
                    hC[k] = 0.0
                    wC[k] = z_bC[k]
                
            VelC = domain.quantities['velocity'].centroid_values
            StageC = domain.quantities['stage'].centroid_values
            ElevC = domain.quantities['elevation'].centroid_values
            HeightC = domain.quantities['height'].centroid_values
            XmomC = domain.quantities['xmomentum'].centroid_values

            
            #for i in range(domain.number_of_elements):
            #    if HeightC[i] < 0.0 :
            #        VelC[i] = 0.0
                
            #HeightC = domain.quantities['height'].centroid_values
            #print 'Difference=',HeightC-hC
            #print 'Difference=',StageC-wC
        

            error_h = (1.0/N)*sum(abs(hC-HeightC))
            error_uh = (1.0/N)*sum(abs(uC*hC - XmomC))
            error_u = (1.0/N)*sum(abs(uC - VelC))
            #print 'len(hC)=',len(hC)
            #print 'N=',N
            print 'Height error measured at centroids = ', error_h
            print 'Momentum error measured at centroids = ', error_uh
            print 'Velocity error measured at centroids = ', error_u
        
            StageV = domain.quantities['stage'].vertex_values
            ElevV = domain.quantities['elevation'].vertex_values
            HeightV = domain.quantities['height'].vertex_values
            XmomV = domain.quantities['xmomentum'].vertex_values
            VelV = domain.quantities['velocity'].vertex_values
        
            u,h,w,z_b,T = analytic_cannal(X.flat,t)
            for k in range(len(h)):
                if h[k] < 0.0:
                    h[k] = 0.0
                    w[k] = z_b[k]
        
            from pylab import plot,title,xlabel,ylabel,legend,savefig,show,hold,subplot
            hold(False)
        
            plot1 = subplot(311)
            plot(X,w,  X,StageV,  X,z_b)
            #plot1.set_xlim([-6000,6000])
            #xlabel('Position')
            ylabel('Stage')
            legend(('Analytical Solution', 'Numerical Solution', 'Canal Bed'),
                   'upper center', shadow=False)
        
            plot2 = subplot(312)
            plot(C,uC*hC, C,XmomC) #(X,u*h,X,XmomV)
            #plot2.set_ylim([-1,25])
            #xlabel('Position')
            ylabel('Momentum')
            legend(('Analytical Solution', 'Numerical Solution'),
                   'upper left', shadow=False)

        

            #h_V = zeros(len(VelV),Float)
            #print 'len(h_V)=',len(h_V)
            #print 'len(VelV)=', len(VelV)
            #print 'h_V=',h_V[:]
            #h_V[:] = HeightV
        
            #for i in range(domain.number_of_elements):
            #    for j in range(2):
            #        if HeightV[i,j] <= 0.0:
            #            VelV[i,j] = 0.0
                    
            #print 'len(h_V)=',len(h_V)
            #print 'le(VelV)=', len(VelV)
            #print 'h_V=',h_V[:]
        
            plot3 = subplot(313)
            plot(C,uC, C,VelC) #(X,u, X,VelV)
            xlabel('Position')
            ylabel('Velocity')
            legend(('Analytical Solution', 'Numerical Solution'),
                   'upper right', shadow=False)
            #print 'domain.order=',domain.order
            #print 'domain.limiter=',domain.limiter
            #print 'domain.time=',domain.time
            #print 'domain.timestepping = ',domain.get_timestepping_method()

            #file = "parabolic_new_"
            #file += str(t)
            #file += ".eps"
            #savefig(file)
            #show()
    finaltime=finaltime + T/16#10.0 #T/20.0
raw_input("Press the return key!")



"""
domain.set_quantity('stage', stage)
domain.set_quantity('elevation',elevation)
domain.set_boundary({'exterior': Reflective_boundary(domain)})

C = domain.centroids
X = domain.vertices
u,h,w,z_b,T = analytic_cannal(X.flat,domain.time)
print 'T = ',T

StageQ = domain.quantities['stage'].vertex_values
XmomQ = domain.quantities['xmomentum'].vertex_values

print 'domain.order=',domain.order
print 'domain.limiter=',domain.limiter
print 'domain.time=',domain.time
print 'domain.timestepping = ',domain.get_timestepping_method()

yieldstep = finaltime = T
t0 = time.time()
while finaltime < T+1.0:
    #yieldstep = finaltime
    for t in domain.evolve(yieldstep = yieldstep, finaltime = finaltime):
        domain.write_time()
        print "integral", domain.quantities['stage'].get_integral()
        if t>= 0.0:        
            uC,hC,wC,z_bC,TC = analytic_cannal(C,t)
                
            VelC = domain.quantities['velocity'].centroid_values
            StageC = domain.quantities['stage'].centroid_values
            ElevC = domain.quantities['elevation'].centroid_values
            HeightC = domain.quantities['height'].centroid_values
            XmomC = domain.quantities['xmomentum'].centroid_values
            

            error_h = cell_len*sum(abs(hC-HeightC))
            error_uh = cell_len*sum(abs(uC*hC - XmomC))
            error_u = cell_len*sum(abs(uC - VelC))

            print 'Height error measured at centroids = ', error_h
            print 'Momentum error measured at centroids = ', error_uh
            print 'Velocity error measured at centroids = ', error_u
        
            StageV = domain.quantities['stage'].vertex_values
            ElevV = domain.quantities['elevation'].vertex_values
            HeightV = domain.quantities['height'].vertex_values
            XmomV = domain.quantities['xmomentum'].vertex_values
            VelV = domain.quantities['velocity'].vertex_values
        
            u,h,w,z_b,T = analytic_cannal(X.flat,t)
        
            from pylab import plot,title,xlabel,ylabel,legend,savefig,show,hold,subplot
            hold(False)
        
            plot1 = subplot(311)
            plot(X,w, X,StageV,  X,ElevV)
            #plot1.set_xlim([-6000,6000])
            #xlabel('Position')
            ylabel('Stage')
            legend(('Analytical Solution', 'Numerical Solution', 'Canal Bed'),
                   'upper center', shadow=False)
        
            plot2 = subplot(312)
            plot(C,uC*hC, C,XmomC)
            #plot2.set_ylim([-1,25])
            #xlabel('Position')
            ylabel('Momentum')
            #legend(('Analytical Solution', 'Numerical Solution'),
            #       'upper right', shadow=False)
        
            plot3 = subplot(313)
            plot(C,uC, C,VelC)
            #xlabel('Position')
            ylabel('Velocity')

            #plot4 = subplot(414)
            #plot(X,h, X,HeightV)
            #xlabel('Position')
            #ylabel('Height')
            #file = "parabolic_2nd_vanleer_"
            #file += str(t)
            #file += ".png"
            #savefig(file)
            #show()
    finaltime=finaltime + T/16.0
raw_input("Press the return key!")
"""
