import os
import time
from math import sqrt, sin, cos, pi, exp
from shallow_water_domain_suggestion3 import *
from Numeric import zeros, Float
from pylab import plot,title,xlabel,ylabel,legend,savefig,show,hold,subplot

print "TEST 1D-SOLUTION I"
L=2000.0
N=8

cell_len=L/N
points=zeros(N+1, Float)
for i in range(N+1):
    points[i]=i*cell_len

domain=Domain(points)
domain.order = 2
domain.set_timestepping_method('rk2')
domain.cfl = 1.0
domain.limiter = "minmod"

def stage(x):
    y=zeros(len(x), Float)
    for i in range(len(x)):
        if 1750 <= x[i] < 2000.0:
            y[i] = 13.0
        else:
            y[i] = 0.0
    return y

def momentum(x):
    y=zeros(len(x), Float)
    for i in range(len(x)):
        if 1500.0 <= x[i] < 1750.0:
            y[i] = -400.0 #-150.0
        else:
            y[i] = 0.0
    return y


def elevation(x):
    y=zeros(len(x), Float)
    for i in range(len(x)):
        if 0 <= x[i] < 1250.0:
            y[i] = 12.0
        elif 1250.0 <= x[i] < 1500.0:
            y[i] = 4.0
        else:
            y[i] = 0.0
    return y

domain.set_quantity('stage',stage)
domain.set_quantity('elevation',elevation)
domain.set_quantity('xmomentum',momentum)

domain.set_boundary({'exterior':Reflective_boundary(domain)})
X=domain.vertices
C=domain.centroids

t0=time.time()
yieldstep=finaltime=10.0
while finaltime < 300.2:
    for t in domain.evolve(yieldstep=yieldstep, finaltime=finaltime):
        domain.write_time()
        print "t=",t
        print "integral", domain.quantities['height'].get_integral()

        VelC = domain.quantities['velocity'].centroid_values
        StageC = domain.quantities['stage'].centroid_values
        ElevC = domain.quantities['elevation'].centroid_values
        HeightC = domain.quantities['height'].centroid_values
        XmomV = domain.quantities['xmomentum'].centroid_values
        
        print "StageC  = ", StageC
        print "XmomC   = ", XmomV
        print "ElevC   = ", ElevC
        print "HeightC = ", HeightC
        print "VelC    = ", VelC
        
        StageQ=domain.quantities['stage'].vertex_values
        MomentumQ=domain.quantities['xmomentum'].vertex_values
        ElevationQ=domain.quantities['elevation'].vertex_values
        VelocityQ=domain.quantities['velocity'].vertex_values

        hold(False)
        plot1 = subplot(311)
        plot(X,StageQ, X,ElevationQ)
        plot1.set_ylim([-1.0,15.5])
        xlabel('Position')
        ylabel('Stage')
        legend( ('Numerical Solution', 'Bed Elevation'), 'upper right', shadow=False)

        plot2 = subplot(312)
        plot(X,MomentumQ)
        #plot2.set_ylim([3.998,4.002])
        legend( ('Numerical Solution', 'for momentum'), 'upper right', shadow=False)
        xlabel('Position')
        ylabel('Xmomentum')

        plot3 = subplot(313)
        plot(X,VelocityQ)
        #plot2.set_ylim([-5.0,30.0])
        legend( ('Numerical Solution', 'for velocity'), 'upper right', shadow=False)
        xlabel('Position')
        ylabel('Velocity')
        
        #file = "island_"
        #file += str(finaltime)
        #file += ".png"
        #savefig(file)
        #show()
        
    print 'That took %.2f seconds'%(time.time()-t0)
    finaltime = finaltime + 10.0
    raw_input("Press return key")
