#include <iostream>
#include <sstream>
#include <boost/shared_ptr.hpp>
#include <SDL.h>
#include "anugavis.hh"
#include "output.hh"

using boost::shared_ptr;
using std::cout;
using std::endl;
using std::ostringstream;

extern "C"
int main(int argc, char *argv[]){
  if(argc != 2){
    ostringstream ss;
    ss << "Usage: " << argv[0] << " sww_file_name";
    output_string(ss.str());
    return 1;
  }
  try{
    AnugaVis anuga(argv[1], 640, 480);
    shared_ptr<HeightQuantity> elevation(new HeightQuantity("elevation", 0, 1,
							    0.5, 0.5, 0.5));
    shared_ptr<HeightQuantity> stage(new HeightQuantity("stage", 0, 1,
							0, 0, 0.8));
    anuga.add_HeightQuantity(elevation);
    anuga.add_HeightQuantity(stage);
    anuga.run();
  }catch(const char *str){
    output_string(string(str));
  }catch(string &str){
    output_string(str);
    return 1;
  }
  return 0;
}
