#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>

typedef float vector[3];

/* Useful vector operations. */
static inline void vadd(const vector v1, const vector v2, vector result){
  int i;
  for(i = 0 ; i < 3 ; i++) result[i] = v1[i] + v2[i];
}

static inline float vlen(const vector v){
  return sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
}

static inline void vscale(const vector v, const float scale, vector result){
  int i;
  for(i = 0 ; i < 3 ; i++) result[i] = scale * v[i];
}

static inline void vsub(const vector v1, const vector v2, vector result){
  int i;
  for(i = 0 ; i < 3 ; i++) result[i] = v1[i] - v2[i];
}

static inline float vdot(const vector v1, const vector v2){
  int i;
  float rv = 0;
  for(i = 0 ; i < 3 ; i++) rv += v1[i] * v2[i];
  return rv;
}

static inline void vcross(const vector v1, const vector v2, vector result){
  result[0] = v1[1] * v2[2] - v1[2] * v2[1];
  result[1] = v1[2] * v2[0] - v1[0] * v2[2];
  result[2] = v1[0] * v2[1] - v1[1] * v2[0];
}

#endif
