"""Script for running a tsunami inundation scenario for Wollongong, NSW, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project_slide_parameters.py
The output sww file is stored in project_slide_parameters.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a tsunami wave generated by s submarine mass failure.

Bridgette Lewis, 2008
"""

#-------------------------------------------------------------------------------
# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
import os
import time
from shutil import copy
from os.path import dirname, basename
from os import mkdir, access, F_OK, sep
import sys

# Related major packages
from anuga.abstract_2d_finite_volumes.util import start_screen_catcher, copy_code_files
from anuga.shallow_water import smf

# Application specific imports
import project_slide_parameters              # Definition of file names and polygons
import project_slide

#-------------------------------------------------------------------------------
# Set up scenario (tsunami_source is a callable object used with set_quantity)
#-------------------------------------------------------------------------------
from smf import slide_tsunami

# effect on a3D and wavelength
length = project_slide_parameters.birubi_length
width = project_slide_parameters.birubi_width
depth = project_slide_parameters.birubi_depth
slope = project_slide_parameters.birubi_slope
thickness = project_slide_parameters.birubi_thickness
alpha = project_slide_parameters.birubi_alpha
x0 = project_slide_parameters.slide_origin_birubi[0]
y0 = project_slide_parameters.slide_origin_birubi[1]

gamma = 1.85
massco = 1.0
dragco = 1.0

# no effect on a3D and wavelength but used in Double Gaussian
dx = 0.01
kappa = 3.
kappad = 0.8

# this doesn't seem to apper anywhere in smf
frictionco = 0.01

# scaling for Double Gaussian function
scale = 30. # Bridgette's fiddle 

fid = open('test.csv','w')
s = 'parameter, wavelength, 3D_amp, eta_min\n'
fid.write(s)

for i in range(30):
    scale = scale - 1.
    #length = length + 10.
    #depth = depth - 10.
    tsunami_source = slide_tsunami(length=length,
                                   width=width,
                                   depth=depth,
                                   slope=slope,
                                   thickness=thickness, 
                                   x0=x0, 
                                   y0=y0, 
                                   alpha=alpha,
				   gamma=gamma,
				   massco=massco,
			           dragco=dragco,
				   frictionco=frictionco,
				   dx=dx,
				   kappa=kappa,
				   kappad=kappad,
 				   scale=scale,
                                   verbose=False)

    #parameter = depth
    #parameter = length
    parameter = scale

    s = '%.6f, %.6f, %.6f, %.6f \n' %(parameter,tsunami_source.wavelength,tsunami_source.a3D,tsunami_source.nmin)
    fid.write(s)

fid.close()
